GarrisonHelper = select(2, ...)
GarrisonHelper.frame = CreateFrame("Frame")
local GH = GarrisonHelper
local addonName = ...
local L = GH.L
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};

local welcomePrinted = false;

GH.buildings = {
	[76] = {76,119,120},
	[93] = {93,125,126},
	[91] = {91,123,124},
	[96] = {96,131,132},
	[95] = {95,129,130},
	[94] = {94,127,128},
	[60] = {60,117,118},
	[90] = {90,121,122},
	
	[24] = {24,25,133},
	[159] = {159,160,161},
	[40] = {40,41,138},
	[111] = {111,144,145},--trading post
	
	[8] = {8,9,10},
	--[162] = {162,163,164},
	[37] = {37,38,39},
	
	[29] = {29,136,137},
	[61] = {61,62,63},
};
function GH:buildingHash(id)--return id of the level 1 version of this building id
	if GH.buildings[id] then
		return id
	else
		for res, arr in pairs(GH.buildings) do
			for i=1,#arr do
				if arr[i]==id then
					return res
				end
			end
		end
	end
	return id
end

-- is NPC with ID id targeted
function GH:isNPC(id, t)
	t = t or "target"
	return UnitExists(t) and tonumber( select( 6, strsplit( "-", UnitGUID(t) ) ), 10 ) == id; 
end
--TODO C_Garrison.GetPlotsForBuilding
function GH:hasBuilding(id, level, hasFollower)
	local ids = {}
	-- id can be any version of this building
	if GH.buildings[GH:buildingHash(id)] then
		ids = GH.buildings[GH:buildingHash(id)]
	else
		ids = {id}
	end
	
	local found = false;
	local res = false;
	local buildings = C_Garrison.GetBuildings()
	if #buildings == 0 then -- may happen, when not initialized yet
		return false
	end
	
	for i = 1, #buildings do
		for j = 1, #ids do
			if buildings[i].buildingID==ids[j] then
				-- GarrBuilding_Armory_3_A GarrBuilding_Armory_3_H
				if tonumber( select( 3, strsplit( "_", buildings[i].texPrefix)), 10) >= level then
					found = true
					res = true
					if hasFollower and not C_Garrison.GetFollowerInfoForBuilding(buildings[i].plotID) then
						-- follwer should be assigned, but is not
						res = false	
					end
					break
				end
			end
		end
		if found then break end
	end
	
	return res
end

function GH:AssignedNPC(id)
	local ids = {}
	-- id can be any version of this building
	if GH.buildings[GH:buildingHash(id)] then
		ids = GH.buildings[GH:buildingHash(id)]
	else
		ids = {id}
	end
	local res = nil;
	local buildings = C_Garrison.GetBuildings()
	for i = 1, #buildings do
		for j = 1, #ids do
			if buildings[i].buildingID==ids[j] then
				res = C_Garrison.GetFollowerInfoForBuilding(buildings[i].plotID)
				break
			end
		end
		if res then break end
	end
	
	return res
end

function events.ADDON_LOADED(self, event, ...)
	local name = ...
	if name==addonName then
		GarrisonHelper_ConfigInit();
		L = GH.L
	end
end

function events.PLAYER_LOGIN()
	if not welcomePrinted then
		welcomePrinted = true;
		C_Timer.After(10, GH_welcomeMessages) -- 10 seconds delay
	end
end

function GH:print(msg, force)
	if GH.db.profile.verbose or force then
		print("|r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF "..tostring(msg));
	end
end

function GH_welcomeMessages()
		if not GH.db.profile.wasconfigured then
			GH:print(L["Type |cFF00FF00/gh|r|cFFFFFFFF to configure several garrison building daily supports."]);
		end
		GH_armoryWelcome()
		GH_jewelcraftingWelcome()
		GH_scribeWelcome()
		GH_alchemyWelcome()
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) events[event](self, event, ...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end

function GH:setActionButton(id, macro, desc, isSpell, iconItemID, hideButton, parent, texture, anchor)
	-- Config button
	GH.btn = GH.btn or CreateFrame("Button", "GH_Button", UIParent, "SecureActionButtonTemplate,ActionButtonTemplate")
	GH.btn:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	GH.btn:SetAttribute("type", "macro");-- click
	GH.btn:SetAttribute("macrotext", macro);
	GH.btn:SetAttribute("type1", "macro"); -- left click
	GH.btn:SetAttribute("macrotext1", macro);
	GH.btn:SetAttribute("type2", "macro");-- right click
	GH.btn:SetAttribute("macrotext2", macro);
	GH.btn:SetSize(100,100)
	GH.btn:SetClampedToScreen(true);
	GH.btn.parent = parent
	GH.btn.icon = GH.btn.icon or GH.btn:CreateTexture(nil, "OVERLAY")
	GH.btn:SetNormalTexture(nil) -- clears border
	GH.btn:SetHighlightTexture(nil) -- clears highlight border
	GH.btn:SetPushedTexture(nil) -- clears pushed button highlight
	GH.btn:UnlockHighlight()
	
	GH.btn.icon:SetAllPoints(true)
	
	if texture then
		
		GH.btn.icon:SetTexture(texture)
	elseif iconItemID then
		GH.btn.icon:SetTexture(GetItemIcon(iconItemID))-- don't know how to set SpellIcon for Milling (for instance)
	elseif id then
		GH.btn.icon:SetTexture(GetItemIcon(id))
	else
		GH.btn.icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
	end
	GH.btn:SetFrameStrata("TOOLTIP")
	--Tooltip
	GH.btn:SetScript("OnEnter", function(self)
		GameTooltip:Hide();
		GameTooltip:SetOwner(self,"ANCHOR_BOTTOMLEFT");
		GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
		if id then
			if isSpell then
				GameTooltip:SetSpellByID(format("%d",id));
			else
				GameTooltip:SetItemByID(format("%d",id));
			end
		end
		GameTooltip:SetClampedToScreen(true);
		GameTooltip:AddLine("\n|r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF:",  1,1,0.5, true)
		GameTooltip:AddLine("|TInterface\\TUTORIALFRAME\\UI-TUTORIAL-FRAME:16:14:0:-1:512:512:10:70:226:306|t " .. L["or"].." |TInterface\\TUTORIALFRAME\\UI-TUTORIAL-FRAME:16:14:0:-1:512:512:10:70:330:410|t" .. " " .. desc .. "\n"..L["<move out> to hide button"], 1, 1, 1, true)
		GameTooltip:Show();
	end);
	GH.btn:SetScript("OnLeave", function(self)
		GameTooltip:Hide();
		GH:destroy()
	end);
	GH.btn:SetScript("OnMouseup", function(self)
		--GH:destroy()
		C_Timer.After(1,function() GH:destroy() end) -- 1 second delay
	end);
	if not hideButton then
		GH.btn:ClearAllPoints()
		if anchor then
			--GH.btn:SetPoint ("CENTER", anchor)
			GH.btn:SetPoint ("BOTTOMLEFT", anchor,"BOTTOMLEFT",-100,-102)
		else
			-- place button at mouse position
			local x, y = GetCursorPosition();
			GH.btn:SetPoint ("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x/UIParent:GetEffectiveScale()-50, y/UIParent:GetEffectiveScale()-50 )
		end
		GH.btn:Show()	
	else
		GH:destroy()
	end
end

function GH:destroy(parent)
	-- actually buttons and frames cannot be destroyed
	if GH.btn then
		if parent and GH.btn.parent and parent~=GH.btn.parent then return end --only destroy, if it is your button
		-- instead, we move them away
		GH.btn:SetClampedToScreen(false);
		GH.btn:SetPoint("BOTTOMLEFT",-9999,-9999)
		GH.btn:Hide()
	end
	if GameTooltip then
		GameTooltip:Hide();
	end
end
function GH_destroy(parent) GH:destroy(parent) end

function GH:getIDFromLink(link)
	if link then
		return tonumber(link:match("item:(%d+)"))
	end
end
--[[
	language independent method to check if you are in a garrison building
]]
function GH:IsCurrentZoneThisBuilding(idOrName)
	SetMapToCurrentZone()
	local zone = GetMinimapZoneText();
	if zone==idOrName then
		return true
	elseif type(idOrName)=="number" then
		local _,buildingName = C_Garrison.GetBuildingInfo(idOrName)
		return zone==buildingName
	else
		return false
	end
end


--[[Phanx, http://www.wowinterface.com/forums/showthread.php?t=46934]]--
GH.QuestTitleFromIdTable = setmetatable({}, { __index = function(t, id)
	GH.MyScanningTooltip = GH.MyScanningTooltip or CreateFrame("GameTooltip", "MyScanningTooltip", UIParent, "GameTooltipTemplate")
	GH.MyScanningTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	GH.MyScanningTooltip:SetHyperlink("quest:"..id)
	local title = MyScanningTooltipTextLeft1:GetText()
	GH.MyScanningTooltip:Hide()
	if title and title ~= RETRIEVING_DATA then
		t[id] = title
		return title
	end
end })
function GH:QuestTitleFromID(id) return GH.QuestTitleFromIdTable[id] end
function GH:QuestLinkFromID(id)
	return "\124cffffff00\124Hquest:"..id..":100\124h["..GH:QuestTitleFromID(id).."]\124h\124r"
end
--[[
	NPC name by id
]]
GH.NpcNameFromIdTable = setmetatable({}, { __index = function(t, id)
	GH.MyScanningTooltip = GH.MyScanningTooltip or CreateFrame("GameTooltip", "MyScanningTooltip", UIParent, "GameTooltipTemplate")
	GH.MyScanningTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	GH.MyScanningTooltip:SetHyperlink(string.format("unit:Creature-0-0000-0-00-%s-0000000000", id ) )
	local title = MyScanningTooltipTextLeft1:GetText()
	GH.MyScanningTooltip:Hide()
	if title and title ~= RETRIEVING_DATA then
		t[id] = title
		return title
	end
end })
function GH:NpcNameFromId(id) return GH.NpcNameFromIdTable[id] end